@@ -10,11 +10,11 @@ HTML::SocialMedia - Put social media links into your website
 
 =head1 VERSION
 
-Version 0.21
+Version 0.22
 
 =cut
 
-our $VERSION = '0.21';
+our $VERSION = '0.22';
 use constant DEFAULTFACEBOOKURL => 'http://connect.facebook.net/en_GB/all.js#xfbml=1';
 
 =head1 SYNOPSIS
@@ -61,52 +61,52 @@ set, such as L<CHI>.
 =cut
 
 sub new {
-	my ($proto, %params) = @_;
+	my $proto = shift;
 
 	my $class = ref($proto) || $proto;
+	my %params = (ref($_[0]) eq 'HASH') ? %{$_[0]} : @_;
 
-	my $lingua;
-	my %args;
-	if($params{twitter}) {
-		# Languages supported by Twitter according to
-		# https://twitter.com/about/resources/tweetbutton
-		$args{supported} = ['en', 'nl', 'fr', 'fr-fr', 'de', 'id', 'il', 'ja', 'ko', 'pt', 'ru', 'es', 'tr'],
-	} else {
-		# TODO: Google plus only supports the languages listed at
-		# http://www.google.com/webmasters/+1/button/index.html
-		require I18N::LangTags::Detect;
-
-		# Facebook supports just about everything
-		my @l = I18N::LangTags::implicate_supers_strictly(I18N::LangTags::Detect::detect());
-
-		if(@l) {
-			$args{supported} = [$l[0]];
+	my $lingua = $params{lingua};
+	unless(defined($lingua)) {
+		my %args;
+		if($params{twitter}) {
+			# Languages supported by Twitter according to
+			# https://twitter.com/about/resources/tweetbutton
+			$args{supported} = ['en', 'nl', 'fr', 'fr-fr', 'de', 'id', 'il', 'ja', 'ko', 'pt', 'ru', 'es', 'tr'],
 		} else {
+			# TODO: Google plus only supports the languages listed at
+			# http://www.google.com/webmasters/+1/button/index.html
+			require I18N::LangTags::Detect;
+
+			# Facebook supports just about everything
+			my @l = I18N::LangTags::implicate_supers_strictly(I18N::LangTags::Detect::detect());
+
+			if(@l) {
+				$args{supported} = [$l[0]];
+			} else {
+				$args{supported} = [];
+			}
+		}
+		if($params{cache}) {
+			$args{cache} = $params{cache};
+		}
+		if($params{logger}) {
+			$args{logger} = $params{logger};
+		}
+		$lingua = $params{lingua} || CGI::Lingua->new(%args);
+		if((!defined($lingua)) && scalar($args{supported})) {
 			$args{supported} = [];
+			$lingua = CGI::Lingua->new(%args);
 		}
 	}
-	if($params{cache}) {
-		$args{cache} = $params{cache};
-	}
-	if($params{logger}) {
-		$args{logger} = $params{logger};
-	}
-	$lingua = $params{lingua} || CGI::Lingua->new(%args);
-	if((!defined($lingua)) && scalar($args{supported})) {
-		$args{supported} = [];
-		$lingua = CGI::Lingua->new(%args);
-	}
 
-	my $self = {
+	return bless {
 		_lingua => $lingua,
 		_twitter => $params{twitter},
 		_twitter_related => $params{twitter_related},
 		_cache => $params{cache},
 		_alpha2 => undef,
-	};
-	bless $self, $class;
-
-	return $self;
+	}, $class;
 }
 
 =head2 as_string
@@ -157,41 +157,37 @@ reddit_button: add a Reddit button
 sub as_string {
 	my ($self, %params) = @_;
 
+	my $lingua = $self->{_lingua};
+
 	unless($self->{_alpha2}) {
-		my $alpha2 = $self->{_lingua}->language_code_alpha2();
+		my $alpha2 = $lingua->language_code_alpha2();
+		my $locale = $lingua->locale();	# Locale::Object::Country
 
 		if($alpha2) {
-			my $salpha2 = $self->{_lingua}->sublanguage_code_alpha2();
-			unless($salpha2) {
-				my $locale = $self->{_lingua}->locale();
-				if($locale) {
-					$salpha2 = $locale->code_alpha2();
-				}
+			my $salpha2 = $lingua->sublanguage_code_alpha2();
+			if((!defined($salpha2)) && defined($locale)) {
+				$salpha2 = $locale->code_alpha2();
 			}
 			if($salpha2) {
 				$salpha2 = uc($salpha2);
 				$alpha2 .= "_$salpha2";
+			} elsif($locale) {
+				my @l = $locale->languages_official();
+				$alpha2 = lc($l[0]->code_alpha2()) . '_' . uc($locale->code_alpha2());
 			} else {
-				my $locale = $self->{_lingua}->locale();
-				if($locale) {
-					my @l = $locale->languages_official();
-					$alpha2 = lc($l[0]->language_code_alpha2()) . '_' . uc($locale->code_alpha2());
-				} else {
-					$alpha2 = undef;
-				}
+				$alpha2 = undef;
 			}
 		}
 
 		unless($alpha2) {
-			my $locale = $self->{_lingua}->locale();
 			if($locale) {
 				my @l = $locale->languages_official();
-				if(scalar(@l) && defined($l[0]->language_code_alpha2())) {
-					$alpha2 = lc($l[0]->language_code_alpha2()) . '_' . uc($locale->language_code_alpha2());
+				if(scalar(@l) && defined($l[0]->code_alpha2())) {
+					$alpha2 = lc($l[0]->code_alpha2()) . '_' . uc($locale->code_alpha2());
 				} else {
 					@l = $locale->languages();
-					if(scalar(@l)) {
-						$alpha2 = lc($l[0]->language_code_alpha2()) . '_' . uc($locale->language_code_alpha2());
+					if(scalar(@l) && defined($l[0]->code_alpha2())) {
+						$alpha2 = lc($l[0]->code_alpha2()) . '_' . uc($locale->code_alpha2());
 					}
 				}
 			}
@@ -206,7 +202,7 @@ sub as_string {
 
 	if($self->{_twitter}) {
 		if($params{twitter_follow_button}) {
-			my $language = $self->{_lingua}->language();
+			my $language = $lingua->language();
 			if(($language eq 'English') || ($language eq 'Unknown')) {
 				$rc = '<a href="http://twitter.com/' . $self->{_twitter} . '" class="twitter-follow-button">Follow @' . $self->{_twitter} . '</a>';
 			} else {
@@ -423,7 +419,7 @@ L<http://search.cpan.org/dist/HTML-SocialMedia/>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2011-2014 Nigel Horne.
+Copyright 2011-2015 Nigel Horne.
 
 This program is released under the following licence: GPL
 
